#!/bin/bash
#
# AHCI Master Boot Record Patch
#
# Written by johnsock
#
# This script is freely distributable.
# Use of this script could muck up your computer. The author accepts no
# responsibility for what might happen when you use it.
#
VERSION=1.0

BACKUP_MBR=./backup.mbr
AHCI_MBR=./ahci.mbr

HASH_XP=d88d4f2dbc2c662db7d7d60ebf9e3e00
HASH_VISTA=12c9d7ff4914c88a7f9eadf9211b861b
HASH_WINDOWS7=118eb70c44cb69284e5d8aa93096831e 
NEW_MESSAGES=42616420706172746974696F6E207461626C6500426164204F53004E6F204F5300434B4A91A5AC
AHCI_CODE=66B890FA0080BAF80C66EFB040BAFC0CEEC3

echo AHCI Master Boot Record Patch $VERSION
if [ "$1" = "" ]; then
	echo Type $0 -h for command line options.
fi
echo
ARGS=`getopt hnf:b:m: $*`
if [ "$?" != 0 ]; then
	exit
fi
set -- $ARGS
for i
do
	case "$i"
	in
		-h)
			echo 'USAGE: '$0' [-h] [-n] [-f <MBR_FILE>] [-b <BACKUP_FILE>] [-m <AHCI_FILE>]';
			echo '       -h               Help'
			echo '       -n               Do not write MBR to disk'
			echo '       -f <MBR_FILE>    Get MBR from MBR_FILE instead of scanning disks'
			echo '       -b <BACKUP_FILE> Specify name of MBR backup file'
			echo '       -m <AHCI_FILE>   Specify name of modified MBR file'
			echo;
			exit;;
		-n)
			NOWRITE="1";
			shift;;
		-f)
			INFILE="$2"; shift;
			shift;;
		-b)
			BACKUP_MBR="$2"; shift;
			shift;;
		-m)
			AHCI_MBR="$2"; shift;
			shift;;
		--)
			shift; break;;
	esac
done

if [ ! "$USER" = "root" ]; then
	echo WARNING: This script must be run as root.
	exit
fi

if [ "$INFILE" = "" ]; then
	echo Scanning drives...
	DRIVELIST=`diskutil list | grep "/dev/disk"`
	for DRIVE in $DRIVELIST;
	do
		dd if=$DRIVE bs=1 skip=510 count=2 2> /dev/null | xxd -p | grep -q -e "55aa" > /dev/null
		if [ "$?" = 0 ]; then
			DEFAULT_DRIVE=$DRIVE
		fi
	done

	if [ "$DEFAULT_DRIVE" != "" ]; then
		echo Windows partition found on $DEFAULT_DRIVE
		read -n1 -p "Do you want to use this drive? (Y/N)" KEYPRESS
		if [ "$KEYPRESS" = 'Y' ] || [ "$KEYPRESS" = 'y' ]; then
			INFILE=$DEFAULT_DRIVE
		fi
		echo
	fi
	if [ "$INFILE" = "" ]; then
		read -p "Enter drive to use: " INFILE
		if [ "$INFILE" = "" ]; then
			echo Aborted.
			exit
		fi
		DEFAULT_DRIVE=$INFILE
	fi
else
	FSIZE=`stat -f %z $INFILE`
	if [ "$FSIZE" != "446" ] && [ "$FSIZE" != "512" ]; then
		echo ERROR: The file $INFILE does not contain a disk signature.
		echo
		exit
	fi
fi
CHECK=`dd if=$INFILE bs=1 skip=401 count=39 2> /dev/null | xxd -p -u -c 39`
if [ "$CHECK" = "$NEW_MESSAGES" ]; then
	echo The MBR on $INFILE appears to already be modified.
	exit
fi
echo Backing up MBR from $INFILE to $BACKUP_MBR
dd if=$INFILE of=$BACKUP_MBR bs=1 count=446 2> /dev/null
if [ "$?" != 0 ]; then
	echo ERROR: Unable to read MBR
	exit
fi
echo MBR backup complete.
echo

HASH=`xxd -p -l 440 $BACKUP_MBR | md5`

if [ "$HASH" = "$HASH_XP" ]; then
	OS=XP
elif [ "$HASH" = "$HASH_VISTA" ]; then
	OS=Vista
elif [ "$HASH" = "$HASH_WINDOWS7" ]; then
	OS=7
fi

if [ "$OS" = "" ]; then
	read -n1 -p "WARNING: Unrecognized OS. Do you want to continue? (Y/N)" KEYPRESS
	if [ "$KEYPRESS" = 'N' ] || [ "$KEYPRESS" = 'n' ]; then
		echo
		exit
	fi
	echo
else
	echo Detected Windows $OS Master Boot Record.
fi

MESSAGE_START='0x1'`xxd -p -s 437 -l 1 $BACKUP_MBR`
x=$MESSAGE_START
while [ "$x" != "0" ]
do
	BYTE=`xxd -p -s $x -l 1 $BACKUP_MBR`
	if [ "$BYTE" = "c3" ]; then
		CODE_START=$x
		break
	fi
	x=$(($x-1))
done

echo `xxd -p -l $CODE_START $BACKUP_MBR`$AHCI_CODE | xxd -r -p | dd ibs=1 obs=401 conv=osync 2> /dev/null > $AHCI_MBR
echo $NEW_MESSAGES | xxd -r -p | dd of=$AHCI_MBR bs=1 seek=401 2> /dev/null
echo `xxd -p -s -6 $BACKUP_MBR` | xxd -r -p | dd of=$AHCI_MBR bs=1 seek=440 2> /dev/null

echo Modified MBR saved to $AHCI_MBR

if [ "$NOWRITE" = "1" ]; then
	exit
fi
if [ "$DEFAULT_DRIVE" = "" ]; then
	read -p "Enter drive for writing modified MBR: " DEFAULT_DRIVE
	if [ "$DEFAULT_DRIVE" = "" ]; then
		echo Aborted.
		exit
	fi
fi

read -n1 -p "Are you ABSOLUTELY sure you want to write the modified MBR to $DEFAULT_DRIVE? (Y/N)" KEYPRESS
if [ "$KEYPRESS" = 'Y' ] || [ "$KEYPRESS" = 'y' ]; then
	TEMP=${DEFAULT_DRIVE/rdisk/disk}
	RAW_DRIVE=${TEMP/disk/rdisk}
	fdisk -y -u -f $AHCI_MBR $RAW_DRIVE 2> /dev/null
	if [ "$?" = 0 ]; then
		echo Modified MBR written to $DEFAULT_DRIVE
	else
		echo ERROR: Unable to write MBR to $DEFAULT_DRIVE
	fi 
fi 
echo

